'''
"Test your memory"
A simple game
for small kids.
'''

VERSION = '0.1.1'

import os, pygame, random, time
import pygame.mixer
import gp2x

from pygame.locals import *

if not pygame.font: print 'Warning, fonts disabled'
if not pygame.mixer: print 'Warning, sound disabled'

run = 1
cpu_list = []
player_list = []
play_var = 0
group = ()
allsprites = None
_font = 'VeraSe.ttf'
color = 240, 235, 219
size = 320, 240
screen = pygame.display.set_mode(size)
pygame.display.set_caption('Test your memory')
background = pygame.Surface(screen.get_size())
pygame.mixer.pre_init(44100,-16,2,4096)
pygame.init()
pygame.mouse.set_visible(0)
num_joysticks = pygame.joystick.get_count()
if num_joysticks > 0:
	stick = pygame.joystick.Joystick(0)
	stick.init() # now we will receive events for the joystick


def load_image(name, colorkey=None):
	fullname = os.path.join('data', name)
	try:
		image = pygame.image.load(fullname)
	except pygame.error, message:
		print 'Cannot load image:', fullname
		raise SystemExit, message
	image = image.convert()
	if colorkey is not None:
		if colorkey is -1:
			colorkey = image.get_at((0,0))
		image.set_colorkey(colorkey, RLEACCEL)
	return image, image.get_rect()

def load_sound(name):
	class NoneSound:
		def play(self):
			pass
	if name != None:
		fullname = os.path.join('data', name)
		soundfile = pygame.mixer.Sound(fullname)
		return soundfile
	else:
		return NoneSound()

def _update(a,b,s):
	a.update()
	s.blit(b,(0,0))
	a.draw(s)
	pygame.display.flip()
	while pygame.time.delay(400) < 400:
		pass

def display_some_text(text,size,x,y,b,u, color=(0,0,0)):
	global _font
	font = pygame.font.Font(os.path.join('data',_font), size)
	font.set_underline(u)
	t = font.render(text, 1, color)
	tpos = t.get_rect()
	tpos.center = (x, y)
	b.blit(t, tpos)

class simple_button(pygame.sprite.Sprite):
	def __init__(self,x,y,pic,screen,background,index,sound,text='',size=0):
		global _font
		pygame.sprite.Sprite.__init__(self)
		self.image, self.rect = load_image(pic)
		self.status = self.down = self.up = 0
		self.ypos = self.rect.top = y
		self.xpos = self.rect.left = x
		self.background = background
		self.index = index
		self.screen = screen
		self.text = text
		self.size = size
		self.sound = load_sound(sound)
		# Draw lines
		self.screen.lock()
		pygame.draw.line(self.background,(0,0,0),(self.xpos+1,self.ypos+self.rect.height)\
,(self.xpos+self.rect.width-1,self.ypos+self.rect.height)) #horizontal
		pygame.draw.line(self.background,(0,0,0),(self.xpos+self.rect.width,self.ypos+1)\
,(self.xpos+self.rect.width,self.ypos+self.rect.height)) #vertical
		self.screen.unlock()
		# Display some text
		if text != '':
			font = pygame.font.Font(os.path.join('data',_font), self.size)
			self.font = font.render(self.text, 1, (10, 10, 10))
			self.font_rect = self.font.get_rect()
			self.font_rect.center = self.rect.center[0] - self.xpos, self.rect.center[1] - self.ypos
			self.image.blit(self.font, self.font_rect)
	def press(self):
		self.sound.play()
		self.rect.inflate_ip(-2,-2)
		self.status = 1
	def unpress(self):
		self.rect.inflate_ip(2,2)
		self.status = 0
	def update(self):
		if self.down:
			self.press()
			self.down = 0
		elif self.up:
			self.unpress()
			self.up = 0
	def is_focused(self):
		return self.rect.collidepoint(pygame.mouse.get_pos())

def begin():
	global run, group, allsprites, color ,size, screen, background, play_var
	play_var = 0

	background = background.convert()
	background.fill(color)
	screen.blit(background, (0, 0))
	pygame.display.flip()

	musicfile = os.path.join('data','scn15_1.ogg')
	pygame.mixer.music.load(musicfile)
	pygame.mixer.music.play(-1)

	button1 = simple_button(32,90,'tymbutton.png',screen,background,0,None,'Play!! (A)',12)
	button2 = simple_button(200,90,'tymbutton.png',screen,background,0,None,'EXIT (B)',12)
	group = (button1,button2)
	allsprites = pygame.sprite.RenderPlain(group)

	display_some_text('Test your memory',18,145,50,background,1)
	display_some_text('Make a choice',12,155,200,background,1)

	while run:
		for event in pygame.event.get():
			if event.type == QUIT:
				run = 0
			elif event.type == pygame.JOYBUTTONDOWN:
				if event.button == gp2x.BUTTON_A:
					button1.press()
				elif event.button == gp2x.BUTTON_B:
					button2.press()
			elif event.type == pygame.JOYBUTTONUP:
				if button1.status == 1:
					button1.unpress()
					play_var = 1
					run = 0
				elif button2.status == 1:
					button2.unpress()
					run = 0

		screen.blit(background,(0,0))
		allsprites.draw(screen)
		pygame.display.flip()

def cpu_play():
	global run, cpu_list, player_list,group, allsprites, color ,size, screen, background, play_var
	run = 1
	play_var = 0

	color = 240, 235, 219
	background.fill(color)

	cpu_list.append(random.randint(1,4))

	button1 = simple_button(110,15,'tymb1.png',screen,background,1,'n522.wav')
	button2 = simple_button(110,170,'tymb2.png',screen,background,2,'n544.wav')
	button3 = simple_button(15,75,'tymb3.png',screen,background,3,'n533.wav')
	button4 = simple_button(240,75,'tymb4.png',screen,background,4,'n555.wav')
	group = (button1,button2,button3,button4)
	allsprites = pygame.sprite.RenderPlain(group)
	display_some_text('Go !',30,160,120,background,0)

	for c in range(2):
		_update(allsprites,background,screen)

	while run:
		for event in pygame.event.get():
			if event.type == QUIT:
				run = 0
				play_var = 1
		for number in cpu_list:
			for button in allsprites:
				if button.index == number:
					button.down = 1
					_update(allsprites,background,screen)
					button.up = 1
					_update(allsprites,background,screen)
		run = 0

def player_play():
	global run, cpu_list, player_list, group, allsprites, color ,size, screen, background, play_var
	run = 1
	play_var = 0

	color = 240, 235, 219
	background.fill(color)

	button1 = simple_button(110,15,'tymb1.png',screen,background,1,'n522.wav')
	button2 = simple_button(110,170,'tymb2.png',screen,background,2,'n544.wav')
	button3 = simple_button(15,75,'tymb3.png',screen,background,3,'n533.wav')
	button4 = simple_button(240,75,'tymb4.png',screen,background,4,'n555.wav')
	group = (button1,button2,button3,button4)
	allsprites = pygame.sprite.RenderPlain(group)
	display_some_text('Go !',30,160,120,background,0)

	while run:
		for event in pygame.event.get():
			if event.type == QUIT:
				run = 0
				play_var = 1
			elif event.type == pygame.JOYBUTTONDOWN:
				for button in allsprites:
					if (event.button == gp2x.BUTTON_A):
						button = button3
						button.down = 1
					elif (event.button == gp2x.BUTTON_B):
						button = button4
						button4.down = 1
					elif (event.button == gp2x.BUTTON_X):
						button = button2
						button2.down = 1
					elif (event.button == gp2x.BUTTON_Y):
						button = button1
						button1.down = 1
					elif (event.button == gp2x.BUTTON_SELECT):
						run = 0
			elif event.type == pygame.JOYBUTTONUP:
				for button in allsprites:
					if button.status == 1:
						button.up = 1
						player_list.append(button.index)

			if len(cpu_list) == len(player_list):
				run = 0

		allsprites.update()
		screen.blit(background,(0,0))
		allsprites.draw(screen)
		pygame.display.flip()

def after_play():
	global run, cpu_list, player_list, group, allsprites, color ,size, screen, background, play_var
	play_var = 0
	run = 1

	score = str(len(cpu_list)-1)
	
	color = 240, 235, 219
	background.fill(color)

	if not pygame.mixer.music.get_busy():
		musicfile = os.path.join('data','scn15_1.ogg')
		pygame.mixer.music.load(musicfile)
		pygame.mixer.music.play(-1)

	button1 = simple_button(125,100,'tymbutton.png',screen,background,0,None,'Play again(Y)',11)
	button2 = simple_button(125,150,'tymbutton.png',screen,background,0,None,'Credits(B)',12)
	button3 = simple_button(125,200,'tymbutton.png',screen,background,0,None,'Exit(X)',12)
	group = (button1,button2,button3)
	allsprites = pygame.sprite.RenderPlain(group)

	display_some_text('Your score is',30,165,25,background,1)
	display_some_text(score,40,165,70,background,0)

	while run:
		for event in pygame.event.get():
			if event.type == QUIT:
				run = 0
			elif event.type == pygame.JOYBUTTONDOWN:
				if (event.button == gp2x.BUTTON_Y):
					button1.press()
				elif (event.button == gp2x.BUTTON_B):
					button2.press()
				elif (event.button == gp2x.BUTTON_X):
					button3.press()
			elif event.type == pygame.JOYBUTTONUP:
				if button1.status == 1:
					button1.unpress()
					play_var = 1
					run = 0
				elif button2.status == 1:
					button2.unpress()
					play_var = 2
					run = 0
				elif button3.status == 1:
					button3.unpress()
					play_var = 3
					run = 0

		screen.blit(background,(0,0))
		allsprites.draw(screen)
		pygame.display.flip()

def _credits():
	global run, group, allsprites, color ,size, screen, background, play_var
	run = 1
	a = ''
	y = 5
	x = 5
	play_var = 0
	color = 240, 235, 219
	background.fill(color)

	text_file = os.path.join('data','Test your memory info.txt')
	in_file = open(text_file,"r")
	text = in_file.read()
	in_file.close()

	for t in text:
		if t == '\n':
			y = y + 10
			x = 5
		else:
			display_some_text(t,10,x,y,background,0)
		x = x + 8
		screen.blit(background,(0,0))
		pygame.display.flip()

	button1 = simple_button(120,190,'tymbutton.png',screen,background,0,None,'Nice (X)',14)
	group = (button1)
	allsprites = pygame.sprite.RenderPlain(group)

	while run:
		for event in pygame.event.get():
			if event.type == QUIT:
				run = 0
			elif event.type == pygame.JOYBUTTONDOWN:
				if (event.button == gp2x.BUTTON_X):
					button1.press()
			elif event.type == pygame.JOYBUTTONUP:
				if button1.status == 1:
					button1.unpress()
					play_var = 1
					run = 0
	
		screen.blit(background,(0,0))
		allsprites.draw(screen)
		pygame.display.flip()

def main():
	global play_var, cpu_list, player_list, group, allsprites
	begin()
	if play_var == 1:
		pygame.mixer.music.stop()
		allsprites.remove(group)
		main_2()
	else:
		pygame.quit()

def main_2():
	global play_var, cpu_list, player_list, group, allsprites
	cpu_play()
	if play_var == 1:
		pygame.quit()
	else:
		allsprites.remove(group)
		player_play()
		if cpu_list == player_list:
			player_list = []
			while pygame.time.delay(600) < 600:
				pass
			main_2()
		elif play_var == 1:
			pygame.quit()
		elif cpu_list != player_list:
			allsprites.remove(group)
			main_3()
		elif play_var == 0:
			allsprites.remove(group)
			main_2()

def main_3():
	global play_var, cpu_list, player_list, group, allsprites
	after_play()
	if play_var == 1:
		pygame.mixer.music.stop()
		allsprites.remove(group)
		cpu_list = []
		player_list = []
		main_2()
	elif play_var == 2:
		allsprites.remove(group)
		_credits()
		if play_var == 1:
			main_3()
		else:
			pygame.quit()
	elif play_var == 3:
		allsprites.remove(group)
		pygame.quit()

if __name__ == '__main__': main()
